"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _react = _interopRequireWildcard(require("react"));

var _propTypes = _interopRequireDefault(require("prop-types"));

var _index = _interopRequireDefault(require("epubjs/lib/index"));

var _style = _interopRequireDefault(require("./style"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _getRequireWildcardCache() { if (typeof WeakMap !== "function") return null; var cache = new WeakMap(); _getRequireWildcardCache = function _getRequireWildcardCache() { return cache; }; return cache; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

global.ePub = _index.default; // Fix for v3 branch of epub.js -> needs ePub to by a global var

class EpubView extends _react.Component {
  constructor(props) {
    super(props);

    _defineProperty(this, "onLocationChange", loc => {
      const _this$props = this.props,
            location = _this$props.location,
            locationChanged = _this$props.locationChanged;
      const newLocation = loc && loc.start;

      if (location !== newLocation) {
        this.location = newLocation;
        locationChanged && locationChanged(newLocation);
      }
    });

    _defineProperty(this, "handleKeyPress", ({
      key
    }) => {
      key && key === "ArrowRight" && this.nextPage();
      key && key === "ArrowLeft" && this.prevPage();
    });

    this.state = {
      isLoaded: false,
      toc: []
    };
    this.viewerRef = /*#__PURE__*/_react.default.createRef();
    this.location = props.location;
    this.book = this.rendition = this.prevPage = this.nextPage = null;
  }

  componentDidMount() {
    this.initBook(true);
    document.addEventListener("keyup", this.handleKeyPress, false);
  }

  initBook(first) {
    const _this$props2 = this.props,
          url = _this$props2.url,
          tocChanged = _this$props2.tocChanged,
          epubInitOptions = _this$props2.epubInitOptions;

    if (this.book) {
      this.book.destroy();
    }

    this.book = new _index.default(url, epubInitOptions);
    this.book.loaded.navigation.then(({
      toc
    }) => {
      this.setState({
        isLoaded: true,
        toc: toc
      }, () => {
        tocChanged && tocChanged(toc);
        this.initReader();
      });
    });
  }

  componentWillUnmount() {
    this.book = this.rendition = this.prevPage = this.nextPage = null;
    document.removeEventListener("keyup", this.handleKeyPress, false);
  }

  shouldComponentUpdate(nextProps) {
    return !this.state.isLoaded || nextProps.location !== this.props.location || nextProps.location !== this.props.location;
  }

  componentDidUpdate(prevProps) {
    if (prevProps.location !== this.props.location && this.location !== this.props.location) {
      this.rendition.display(this.props.location);
    }

    if (prevProps.url !== this.props.url) {
      this.initBook();
    }
  }

  initReader() {
    const toc = this.state.toc;
    const _this$props3 = this.props,
          location = _this$props3.location,
          epubOptions = _this$props3.epubOptions,
          getRendition = _this$props3.getRendition,
          handleKeyPress = _this$props3.handleKeyPress;
    const node = this.viewerRef.current;
    this.rendition = this.book.renderTo(node, _objectSpread({
      contained: true,
      width: "100%",
      height: "100%"
    }, epubOptions));

    this.prevPage = () => {
      this.rendition.prev();
    };

    this.nextPage = () => {
      this.rendition.next();
    };

    this.rendition.on("locationChanged", this.onLocationChange);
    this.rendition.on("keyup", handleKeyPress || this.handleKeyPress);
    getRendition && getRendition(this.rendition);
    this.rendition.display(typeof location === "string" || typeof location === "number" ? location : toc[0].href);
  }

  renderBook() {
    const styles = this.props.styles;
    return /*#__PURE__*/_react.default.createElement("div", {
      ref: this.viewerRef,
      style: styles.view
    });
  }

  render() {
    const isLoaded = this.state.isLoaded;
    const _this$props4 = this.props,
          loadingView = _this$props4.loadingView,
          styles = _this$props4.styles;
    return /*#__PURE__*/_react.default.createElement("div", {
      style: styles.viewHolder
    }, isLoaded && this.renderBook() || loadingView);
  }

}

EpubView.defaultProps = {
  loadingView: null,
  locationChanged: null,
  tocChanged: null,
  styles: _style.default,
  epubOptions: {},
  epubInitOptions: {}
};
EpubView.propTypes = {
  url: _propTypes.default.oneOfType([_propTypes.default.string, _propTypes.default.instanceOf(ArrayBuffer)]),
  loadingView: _propTypes.default.element,
  location: _propTypes.default.oneOfType([_propTypes.default.string, _propTypes.default.number]),
  locationChanged: _propTypes.default.func,
  tocChanged: _propTypes.default.func,
  styles: _propTypes.default.object,
  epubInitOptions: _propTypes.default.object,
  epubOptions: _propTypes.default.object,
  getRendition: _propTypes.default.func,
  handleKeyPress: _propTypes.default.func
};
var _default = EpubView;
exports.default = _default;